unit main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;

type
  TForm1 = class(TForm)
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    procedure Idle( Sender: TObject; var Done: Boolean );
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

uses
  DirectDraw;

var
  lpDD7:            IDirectDraw7        = nil;
  lpPrimarySurface: IDirectDrawSurface7 = nil;
  lpBackBuffer:     IDirectDrawSurface7 = nil;

  lpWallpaper:      IDirectDrawSurface7  = nil;
  lpSprite:         IDirectDrawSurface7  = nil;




//------------------------------------------------------------------------------
// :      InitDirectDraw()
// :    DirectDraw  
//------------------------------------------------------------------------------
function InitDirectDraw( hWnd: HWND ): Boolean;
var
  ddsd2: TDDSURFACEDESC2;
  ddsc2: TDDSCAPS2;
begin
  Result := FALSE;

  if FAILED( DirectDrawCreateEx( nil, lpDD7, IID_IDirectDraw7, nil ) ) then
     Exit;

  lpDD7._AddRef();

  if FAILED( lpDD7.SetCooperativeLevel( hWnd, DDSCL_ALLOWREBOOT or
                                            DDSCL_EXCLUSIVE or
                                            DDSCL_FULLSCREEN ) ) then
     Exit;

  if FAILED( lpDD7.SetDisplayMode( 640, 480, 16, 0, 0 ) ) then
     Exit;

  //   DDSURFACEDESC
  ZeroMemory( @ddsd2, SizeOf( TDDSURFACEDESC2 ) );
  ddsd2.dwSize := SizeOf( TDDSURFACEDESC2 );

  ddsd2.dwFlags := DDSD_CAPS or DDSD_BACKBUFFERCOUNT;
  ddsd2.ddsCaps.dwCaps := DDSCAPS_PRIMARYSURFACE or
                             DDSCAPS_FLIP or
                             DDSCAPS_COMPLEX;
  ddsd2.dwBackBufferCount := 1;

  //    (    )
  if FAILED( lpDD7.CreateSurface( ddsd2, lpPrimarySurface, nil ) ) then
     Exit;

  lpPrimarySurface._AddRef();

  //   DDSCAPS
  ZeroMemory( @ddsc2, SizeOf( TDDSCAPS2 ) );
  ddsc2.dwCaps := DDSCAPS_BACKBUFFER;

  if FAILED( lpPrimarySurface.GetAttachedSurface( ddsc2, lpBackBuffer ) ) then
     Exit;

  lpBackBuffer._AddRef();

  Result := TRUE;
end;




//------------------------------------------------------------------------------
// :      ReleaseDirectDraw()
// :     DirectDraw
//------------------------------------------------------------------------------
procedure ReleaseDirectDraw();
begin
  //    
  if lpBackBuffer <> nil then
  begin
    lpBackBuffer._Release();
    lpBackBuffer := nil;
  end;

  //     
  if lpPrimarySurface <> nil then
  begin
    lpPrimarySurface._Release();
    lpPrimarySurface := nil;
  end;

  // ,     DirectDraw
  if lpDD7 <> nil then
  begin
    lpDD7.RestoreDisplayMode(); //  
    lpDD7._Release();
    lpDD7 := nil;
  end;
end;




//------------------------------------------------------------------------------
// :      CreateSurface()
// :   DirectDraw  
//------------------------------------------------------------------------------
function CreateSurface( var lpSurface: IDirectDrawSurface7; nWidth,
                        nHeight: Integer ): Boolean;
var
  ddsd2: TDDSURFACEDESC2;
begin
  Result := FALSE;

  //   ,  
  if lpSurface <> nil then
  begin
    lpSurface._Release;
    lpSurface := nil;
  end;

  //   DDSURFACEDESC2
  ZeroMemory( @ddsd2, SizeOf( TDDSURFACEDESC2 ) );
  with ddsd2 do
  begin
    dwSize         := SizeOf( TDDSURFACEDESC2 );
    dwFlags        := DDSD_CAPS or DDSD_WIDTH or DDSD_HEIGHT;
    ddsCaps.dwCaps := DDSCAPS_OFFSCREENPLAIN;
    dwWidth        := nWidth;
    dwHeight       := nHeight;
  end;

  //  
  if FAILED( lpDD7.CreateSurface( ddsd2, lpSurface, nil ) ) then
     Exit;

  lpSurface._AddRef;

  Result := TRUE;
end;




//------------------------------------------------------------------------------
// :      CopyImage()
// :     DirectDraw
//------------------------------------------------------------------------------
function CopyImage( var lpSurface: IDirectDrawSurface7; hImage: HBITMAP
  ): Boolean;
var
  destDC,          // -
  srcDC:  HDC;     // -
  bm:     BITMAP;  // BITMAP  windows.pas
  hObj:   HGDIOBJ; //  GDI
begin
  Result := FALSE;

  //   ,    
  srcDC := CreateCompatibleDC( 0 );

  //     -  
  if srcDC = 0 then Exit;

  //    
  hObj := SelectObject( srcDC, hImage );
  if ( hObj = 0 ) or ( hObj = GDI_ERROR ) then
     Exit;

  //   ,    BITMAP
  if GetObject( hImage, SizeOf( bm ), @bm ) = 0 then
     Exit;

  //     
  if not CreateSurface( lpSurface, bm.bmWidth, bm.bmHeight ) then
     Exit;

  //  GDI-   
  if SUCCEEDED( lpSurface.GetDC( destDC ) ) then
  begin
    // BtlBlt-   (    DirectDraw)
    BitBlt( destDC, 0, 0, bm.bmWidth, bm.bmHeight, srcDC, 0, 0, SRCCOPY );
    lpSurface.ReleaseDC( destDC );
  end
  else Exit;

  //      GDI
  DeleteObject( hObj );
  DeleteDC( srcDC );

  Result := TRUE;
end;




//------------------------------------------------------------------------------
// :      ()
// :
//------------------------------------------------------------------------------
procedure TForm1.Idle( Sender: TObject; var Done: Boolean );
var
  rRect: TRect;
begin
  SetRect( rRect, 0, 0, 640, 480 );
  lpBackBuffer.BltFast(0, 0, lpWallpaper, @rRect, DDBLTFAST_WAIT );

  SetRect( rRect, 0, 0, 256, 256 );
  lpBackBuffer.BltFast(0, 0, lpSprite, @rRect, DDBLTFAST_WAIT );

  lpPrimarySurface.Flip(nil, DDFLIP_WAIT);

  Done := FALSE;
end;

//------------------------------------------------------------------------------
// :      ()
// :
//------------------------------------------------------------------------------
procedure TForm1.FormCreate(Sender: TObject);
var
  hImage: HBITMAP;
begin
  if not InitDirectDraw( Form1.Handle ) then
  begin
    MessageBox( Form1.Handle, '   DirectDraw!' + #13 +
                ',  640x480x16   .',
                '!', MB_ICONHAND );
    Form1.Close();
  end;

  hImage := LoadImage( 0, 'Wallpaper.bmp', IMAGE_BITMAP, 0, 0, LR_LOADFROMFILE or
                                                               LR_CREATEDIBSECTION );

  if hImage = 0 then Form1.Close();

  if not CopyImage( lpWallpaper, hImage ) then Form1.Close();

  hImage := LoadImage( 0, 'sprite.bmp', IMAGE_BITMAP, 0, 0, LR_LOADFROMFILE or
                                                            LR_CREATEDIBSECTION );
  if not CopyImage( lpSprite, hImage ) then Form1.Close();

  Application.OnIdle := Idle;
end;




//------------------------------------------------------------------------------
// :      ()
// :
//------------------------------------------------------------------------------
procedure TForm1.FormDestroy(Sender: TObject);
begin
  ReleaseDirectDraw();
end;

end.
