(******************************************************************************
 *                                                                            *
 *      ,  2002                                 *
 *                                                                            *
 *  :       main.pas                                                      *
 *  :                  *
 *                                                                            *
 ******************************************************************************)
unit main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, StdCtrls,
  ExtCtrls, ComCtrls;

type
  TForm1 = class(TForm)
    gb1: TGroupBox;
    gb2: TGroupBox;
    gb3: TGroupBox;
    lbInfo: TLabel;
    lb1: TLabel;
    lb2: TLabel;
    lbXAxis: TLabel;
    lbYAxis: TLabel;
    lbIndex: TLabel;
    lbButtons: TLabel;
    btnClose: TButton;
    imView: TImage;
    Bevel1: TBevel;
    gb4: TGroupBox;
    Label1: TLabel;
    Label2: TLabel;
    tbDeadZone: TTrackBar;
    tbSaturation: TTrackBar;
    procedure btnCloseClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure tbDeadZoneChange(Sender: TObject);
    procedure tbSaturationChange(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    procedure Idle( Sender: TObject; var Done: Boolean );
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

uses
  DirectInput8;




//------------------------------------------------------------------------------
//    
//------------------------------------------------------------------------------
var
  lpDI8:          IDirectInput8       = nil;
  lpDIGameDevice: IDirectInputDevice8 = nil;




//------------------------------------------------------------------------------
// :      EnumJoysticksCallback()
// :      
//------------------------------------------------------------------------------
function EnumJoysticksCallback( var lpddi: TDIDeviceInstanceA; pvRef: Pointer ):
   Integer; stdcall;
var
  didc: TDIDEVCAPS;
begin
  Result := DIENUM_CONTINUE;

  //    GUID       
  if FAILED( lpDI8.CreateDevice( lpddi.guidInstance, lpDIGameDevice, nil ) ) then
     Exit;
  lpDIGameDevice._AddRef;

  //    (   )
  case GET_DIDEVICE_TYPE( lpddi.dwDevType ) of
    DI8DEVTYPE_JOYSTICK: Form1.lbInfo.Caption := ' ';
    DI8DEVTYPE_GAMEPAD:  Form1.lbInfo.Caption := ' ';
    DI8DEVTYPE_DRIVING:  Form1.lbInfo.Caption := ' ';
    DI8DEVTYPE_FLIGHT:   Form1.lbInfo.Caption := ' ';
  end;

  //   TDIDEVCAPS,     
  //     
  ZeroMemory( @didc, SizeOf( TDIDEVCAPS ) );
  didc.dwSize := SizeOf( TDIDEVCAPS );

  //    
  if FAILED( lpDIGameDevice.GetCapabilities( didc ) ) then
     Exit;

  //   
  Form1.lbInfo.Caption := Form1.lbInfo.Caption + lpddi.tszProductName + ' (' +
                          IntToStr( didc.dwAxes ) + ' , '+
                          IntToStr( didc.dwButtons ) + ' ).';
  Result := DIENUM_STOP;
end;




//------------------------------------------------------------------------------
// :      CalibrateGameDevice()
// :    
//------------------------------------------------------------------------------
function CalibrateGameDevice( dwDeadZone, dwSaturation: DWORD ): Boolean;
var
  diprgn: TDIPROPRANGE;
  diprdw: TDIPROPDWORD;
begin
  Result := FALSE;

  //   TDIPROPRANGE
  ZeroMemory( @diprgn, SizeOf( TDIPROPRANGE ) );
  diprgn.diph.dwSize       := SizeOf( TDIPROPRANGE );
  diprgn.diph.dwHeaderSize := SizeOf( TDIPROPHEADER );
  diprgn.diph.dwHow        := DIPH_BYOFFSET;

  //      X
  diprgn.diph.dwObj := DIJOFS_X;
  diprgn.lMin       := -1000;
  diprgn.lMax       := +1000;
  //  
  if FAILED( lpDIGameDevice.SetProperty( DIPROP_RANGE, diprgn.diph ) ) then
     Exit;

  //      Y
  diprgn.diph.dwObj := DIJOFS_Y;
  diprgn.lMin       := -1000;
  diprgn.lMax       := +1000;
  //  
  if FAILED( lpDIGameDevice.SetProperty( DIPROP_RANGE, diprgn.diph ) ) then
     Exit;

  //----------------------------------------------------------------------------
  //   TDIPROPDWORD
  ZeroMemory( @diprdw, SizeOf( TDIPROPDWORD ) );
  diprdw.diph.dwSize       := SizeOf( TDIPROPDWORD );
  diprdw.diph.dwHeaderSize := SizeOf( TDIPROPHEADER );
  diprdw.diph.dwHow        := DIPH_BYOFFSET;
  diprdw.dwData            := dwDeadZone; //  

  //      X
  diprdw.diph.dwObj := DIJOFS_X;
  if FAILED( lpDIGameDevice.SetProperty( DIPROP_DEADZONE, diprdw.diph ) ) then
     Exit;

  //      Y
  diprdw.diph.dwObj := DIJOFS_Y;
  if FAILED( lpDIGameDevice.SetProperty( DIPROP_DEADZONE, diprdw.diph ) ) then
     Exit;

  //----------------------------------------------------------------------------
  diprdw.dwData := dwSaturation; //  

  //      X
  diprdw.diph.dwObj := DIJOFS_X;
  if FAILED( lpDIGameDevice.SetProperty( DIPROP_SATURATION, diprdw.diph ) ) then
     Exit;

  //      Y
  diprdw.diph.dwObj := DIJOFS_Y;
  if FAILED( lpDIGameDevice.SetProperty( DIPROP_SATURATION, diprdw.diph ) ) then
     Exit;

  Result := TRUE;
end;




//------------------------------------------------------------------------------
// :      InitDirectInput()
// :    DirectInput  
//------------------------------------------------------------------------------
function InitDirectInput( hWnd: HWND ): Boolean;
begin
  Result := FALSE;

  //   DirectInput
  if FAILED( DirectInput8Create( GetModuleHandle( 0 ), DIRECTINPUT_VERSION,
                                 IID_IDirectInput8, lpDI8, nil ) ) then
     Exit;
  lpDI8._AddRef();

  //       
  if FAILED( lpDI8.EnumDevices( DI8DEVCLASS_GAMECTRL, EnumJoysticksCallback, nil,
                                DIEDFL_ATTACHEDONLY ) ) then
     Exit;

  // ,     
  if lpDIGameDevice = nil then
     Exit;

  //     " " -  
  //   ,       -
  //       .    DInput,   
  //   DIJOYSTATE2   GetDeviceState()
  if FAILED( lpDIGameDevice.SetDataFormat( @c_dfDIJoystick ) ) then
     Exit;

  //    -     
  if FAILED( lpDIGameDevice.SetCooperativeLevel( hWnd, DISCL_BACKGROUND or
                                                       DISCL_EXCLUSIVE ) ) then
     Exit;

  if not CalibrateGameDevice( 0, 6000 ) then
     Exit;

  //   
  lpDIGameDevice.Acquire();

  Result := TRUE;
end;




//------------------------------------------------------------------------------
// :      ReleaseDirectInput()
// :    DirectInput
//------------------------------------------------------------------------------
procedure ReleaseDirectInput();
begin
  //       
  if lpDIGameDevice <> nil then
  begin
    lpDIGameDevice.Unacquire(); //  
    lpDIGameDevice._Release();
    lpDIGameDevice := nil;
  end;

  //    DirectInput ( )
  if lpDI8 <> nil then
  begin
    lpDI8._Release();
    lpDI8 := nil;
  end;
end;




//------------------------------------------------------------------------------
// :      UpdateGameDeviceState()
// :        
//------------------------------------------------------------------------------
function UpdateGameDeviceState(): Boolean;
var
  js2: TDIJOYSTATE;
  i,
  nXPos,
  nYPos:  Integer;

  hr:    HRESULT;
begin
  Result := FALSE;

  //   
  if FAILED( lpDIGameDevice.Poll() ) then
  begin
    //    
    hr := lpDIGameDevice.Acquire();
    // ,    (  -  )
    while hr = DIERR_INPUTLOST do hr := lpDIGameDevice.Acquire();

    Result := TRUE;
    Exit;
  end;

  //    ,     js2
  if FAILED( lpDIGameDevice.GetDeviceState( SizeOf( TDIJOYSTATE), @js2 ) ) then
     Exit;

  //    
  with Form1 do
  begin
    lbXAxis.Caption := IntToStr( js2.lX );
    lbYAxis.Caption := IntToStr( js2.lY );

    lbButtons.Caption := '';

    //      
    for i := 0 to 127 do // DirectInput8   128 
    begin
      if js2.rgbButtons[ i ] = $80 then //   
      if i <= 9 then lbButtons.Caption := lbButtons.Caption + Format( '0%d ', [ i + 1 ] )
                else lbButtons.Caption := lbButtons.Caption + Format( '%d ', [ i + 1 ] );
    end;

    //     
    nXPos := Round( js2.lX / 22 ) + 50;
    nYPos := Round( js2.lY / 22 ) + 50;

    //  
    with imView.Canvas do
    begin
      //   
      Brush.Color := clWhite;
      FillRect( Canvas.ClipRect );

      //  
      MoveTo( nXPos - 4, nYPos );
      LineTo( nXPos + 5, nYPos );

      MoveTo( nXPos, nYPos + 4 );
      LineTo( nXPos, nYPos - 5 );
    end;
  end;

  Result := TRUE;
end;




//------------------------------------------------------------------------------
// :      TForm1.Idle()
// :     
//------------------------------------------------------------------------------
procedure TForm1.Idle( Sender: TObject; var Done: Boolean );
begin
  //      
  if not UpdateGameDeviceState() then
  begin
    MessageBox( Form1.Handle, '  !',
                '!', MB_ICONHAND);
    Form1.Close();
  end;

  Done := FALSE;
end;




//------------------------------------------------------------------------------
// :      TForm1.FormCreate()
// :   DirectInput   
//------------------------------------------------------------------------------
procedure TForm1.FormCreate(Sender: TObject);
begin
  if not InitDirectInput( Form1.Handle ) then
  begin
    MessageBox( Form1.Handle, '   DirectInput!' + #13 +
                ',         .',
                '!', MB_ICONHAND);
    ReleaseDirectInput();
    Halt;
  end;

  //   Idle-.  TTimer   
  //    DirectInput.
  Application.OnIdle := Idle;
end;




//------------------------------------------------------------------------------
// :      TForm1.tbDeadZoneChange()
// :      
//------------------------------------------------------------------------------
procedure TForm1.tbDeadZoneChange(Sender: TObject);
begin
  CalibrateGameDevice( tbDeadZone.Position * 2000, tbSaturation.Position * 2000 );
end;



//------------------------------------------------------------------------------
// :      TForm1.tbSaturationChange()
// :      
//------------------------------------------------------------------------------
procedure TForm1.tbSaturationChange(Sender: TObject);
begin
  CalibrateGameDevice( tbDeadZone.Position * 2000, tbSaturation.Position * 2000 );
end;




//------------------------------------------------------------------------------
// :      TForm1.btnCloseClick()
// :  
//------------------------------------------------------------------------------
procedure TForm1.btnCloseClick(Sender: TObject);
begin
  Form1.Close();
end;




//------------------------------------------------------------------------------
// :      TForm1.FormDestroy()
// :      
//------------------------------------------------------------------------------
procedure TForm1.FormDestroy(Sender: TObject);
begin
   ReleaseDirectInput();
end;

end.
